@@ -117,24 +117,40 @@ installAndLoad <- function(pkgpath, install_dir = tempfile())
     dir.create(libdir <- file.path(install_dir, "lib"))
     file.create(stderr <- file.path(install_dir, "install.stderr"))
 
-    r_libs_user_old <- Sys.getenv("R_LIBS_USER")
-    on.exit(do.call("Sys.setenv", list(R_LIBS_USER=r_libs_user_old)))
     r_libs_user <- paste(c(libdir, .libPaths()), collapse=.Platform$path.sep)
-    Sys.setenv(R_LIBS_USER=r_libs_user)
+    lpath <- paste0("--library=", libdir)
+    res <- callr::rcmd_safe(
+        "INSTALL",
+        c(
+            "--no-test-load", "--use-vanilla", lpath, pkgpath
+        ),
+        env = c(callr::rcmd_safe_env(), R_LIBS_USER = r_libs_user)
+    )
 
-    rcmd <- file.path(Sys.getenv("R_HOME"), "bin", "R")
-    args <- sprintf("--vanilla CMD INSTALL --no-test-load --library=%s %s",
-                    libdir, shQuote(pkgpath))
-    res <- .run_r_command(cmd = rcmd, args = args, stderr = stderr)
-    if (res) {
+    if (!identical(res[["status"]], 0L)) {
         handleError(pkgpath, " must be installable.")
     }
     pkgname <- .getPackageName(pkgpath)
-    args <- sprintf(
-        "--vanilla -e 'library(%s)'", pkgname
+    res <- callr::r(
+        function(pkgname, libdir) {
+            tryCatch({
+                loadNamespace(
+                    package = pkgname, lib.loc = libdir
+                )
+                TRUE
+            }, error = function(e) {
+                FALSE
+            })
+        },
+        args = list(pkgname = pkgname, libdir = libdir),
+        libpath = libdir,
+        cmdargs = c(
+            "--no-save", "--no-restore", "--no-site-file",
+            "--no-init-file", "--no-environ"
+        ),
+        env = c(callr::rcmd_safe_env(), R_LIBS_USER = r_libs_user)
     )
-    res <- .run_r_command(cmd = rcmd, args = args, stderr = stderr)
-    if (res) {
+    if (!res) {
         handleError(pkgpath, " must be loadable.")
     }
     install_dir
@@ -191,16 +207,49 @@ get_deprecated_status_db_url <- function(version) {
     )
 }
 
+#' @importFrom BiocFileCache BiocFileCache bfcquery bfcneedsupdate bfcdownload
+#'   bfcrpath
 get_status_file_cache <- function(url) {
     cache <- tools::R_user_dir("BiocCheck", "cache")
-    bfc <- BiocFileCache::BiocFileCache(cache, ask = FALSE)
-    rpath <- BiocFileCache::bfcrpath(
+    bfc <- BiocFileCache(cache, ask = FALSE)
+
+    bquery <- bfcquery(bfc, url, "rname", exact = TRUE)
+    if (identical(nrow(bquery), 1L) && bfcneedsupdate(bfc, bquery[["rid"]]))
+        bfcdownload(x = bfc, rid = bquery[["rid"]], rtype = "web", ask = FALSE)
+
+    bfcrpath(
         bfc, rnames = url, exact = TRUE, download = TRUE, rtype = "web"
     )
-    update <- BiocFileCache::bfcneedsupdate(bfc, names(rpath))
-    if (update)
-        BiocFileCache::bfcdownload(bfc, names(rpath), ask = FALSE)
-    rpath
+}
+
+.STATUS_FILE_FIELDS <- c(
+    "Package", "Version", "Maintainer", "MaintainerEmail",
+    "PackageStatus", "UnsupportedPlatforms"
+)
+
+.SENTINEL_PACKAGE_STATUS <- matrix(
+    ncol = length(.STATUS_FILE_FIELDS),
+    dimnames = list(NULL, .STATUS_FILE_FIELDS)
+)
+
+.try_read_dcf <- function(file) {
+    pkg_status <- try({
+        read.dcf(
+            file, all = TRUE, fields = .STATUS_FILE_FIELDS
+        )
+    }, silent = TRUE)
+    if (is(pkg_status, "try-error"))
+        .SENTINEL_PACKAGE_STATUS
+    else
+        pkg_status
+}
+
+get_status_from_dcf <- function(status_file) {
+    pkg_status <- .try_read_dcf(status_file)
+    is_deprecated <- pkg_status[, "PackageStatus"] == "Deprecated" &
+        !is.na(pkg_status[, "PackageStatus"])
+    names(is_deprecated) <- pkg_status[, "Package"]
+    is_deprecated
 }
 
 get_deprecated_status <- function(version) {
@@ -208,11 +257,7 @@ get_deprecated_status <- function(version) {
         version <- BiocManager:::.version_bioc(version)
     status_file_url <- get_deprecated_status_db_url(version)
     status_file <- get_status_file_cache(status_file_url)
-    pkg_status <- read.dcf(status_file, all = TRUE)
-    is_deprecated <- pkg_status[, "PackageStatus"] == "Deprecated" &
-        !is.na(pkg_status[, "PackageStatus"])
-    names(is_deprecated) <- pkg_status[, "Package"]
-    is_deprecated
+    get_status_from_dcf(status_file)
 }
 
 getAllDeprecatedPkgs <- function()
@@ -221,7 +266,7 @@ getAllDeprecatedPkgs <- function()
     ## deprecated packages rather than using the VIEWS files
     deps_release <- get_deprecated_status("release")
     deps_devel <- get_deprecated_status("devel")
-    
+
     union(
         names(deps_release[deps_release]),
         names(deps_devel[deps_devel])
@@ -248,13 +293,14 @@ parseFile <- function(infile, pkgdir) {
         outfile <- file.path(parse_dir, "parseFile.tmp")
         suppressWarnings(suppressMessages(
             capture.output({
-                knitr::purl(
-                    input=infile, output=outfile, documentation=0L
+                try_purl_or_tangle(
+                    input=infile, output=outfile, quiet = TRUE, documentation=0L
                 )
             })
         ))
     } else if (grepl("\\.Rd$", infile, TRUE)) {
-        rd <- tools::parse_Rd(infile)
+        uses_rd_pack <- .usesRdpack(pkgdir)
+        rd <- .parse_Rd_pack(infile, usesRdpack = uses_rd_pack)
         outfile <- file.path(parse_dir, "parseFile.tmp")
         code <- capture.output(tools::Rd2ex(rd))
         writeLines(code, con=outfile, sep="\n")
@@ -312,30 +358,31 @@ findSymbolInParsedCode <- function(parsedCode, pkgname, symbolName,
                 if (grepl("\\.R$", name, ignore.case=TRUE))
                     handleMessage(sprintf(
                         "Found %s%s in %s (line %s, column %s)", symbolName,
-                        parens, getDirFile(name), x[i,1], x[i,2]))
+                        parens, .getDirFiles(name), x[i,1], x[i,2]))
                 else
                     handleMessage(sprintf(
                         "Found %s%s in %s", symbolName, parens,
-                        getDirFile(name))) # FIXME test this
+                        .getDirFiles(name))) # FIXME test this
             }
         }
     }
     length(matches) # for tests
 }
 
-getDirFile <- function(fpath) {
-    if (!identical(length(fpath), 1L))
-        stop("<internal> 'fpath' input must be a scalar character")
-    if (nzchar(fpath) && !is.na(fpath))
-        fpath <- file.path(basename(dirname(fpath)), basename(fpath))
-
-    fpath
+.getDirFiles <- function(fpaths) {
+    if (!BiocBaseUtils::isCharacter(fpaths, zchar = TRUE, na.ok = TRUE))
+        stop("<internal> 'fpaths' input must be a character vector")
+    vapply(fpaths, function(fpath) {
+        if (nzchar(fpath) && !is.na(fpath))
+            fpath <- file.path(basename(dirname(fpath)), basename(fpath))
+        fpath
+    }, character(1L))
 }
 
 .getTokenTextCode <- function(parsedf, token, text, lookback = character(0)) {
     cond <- parsedf$token %in% token & parsedf$text %in% text
     if (length(lookback))
-        cond <- cond & parsedf$token[which(cond) - 1] != shQuote(lookback)
+        cond[cond] <- parsedf$token[which(cond) - 1] != lookback
     parsedf[
         cond,
         c("line1", "col1", "token", "text"),
@@ -380,6 +427,9 @@ findSymbolsInParsedCode <-
     }
 
     matches <- Filter(nrow, matches)
+    if (!length(matches))
+        return(character(0L))
+
     matches <- lapply(names(matches), function(nm) {
         dframe <- matches[[nm]]
         dframe[["text"]] <- paste0(dframe$text,
@@ -392,10 +442,11 @@ findSymbolsInParsedCode <-
         function(...) rbind.data.frame(..., make.row.names = FALSE),
         matches
     )
+    matches[] <- lapply(matches, as.character)
     apply(matches, 1L, function(rowdf) {
         fmttxt <- "%s (line %s, column %s)"
         formt <- if (fun) paste0(rowdf["text"], " in ", fmttxt) else fmttxt
-        sprintf(formt, getDirFile(rowdf["filename"]),
+        sprintf(formt, .getDirFiles(rowdf["filename"]),
             rowdf["line1"], rowdf["col1"]
         )
     })
@@ -425,7 +476,7 @@ getMaintainerEmail <- function(pkgdir)
     dcf <- read.dcf(file.path(pkgdir, "DESCRIPTION"))
     if ("Maintainer" %in% colnames(dcf))
     {
-        m <- dcf[, "Maintainer"]
+        m <- unname(dcf[, "Maintainer"])
         ret <- regexec("<([^>]*)>", m)[[1]]
         ml <- attr(ret, "match.length")
         email <- substr(m, ret[2], ret[2]+ml[2]-1)
@@ -438,7 +489,7 @@ getMaintainerEmail <- function(pkgdir)
         tryCatch(people <- eval(pp, env),
             error=function(e) {
                 # could not parse Authors@R
-                return()
+                return(NULL)
             })
         for (person in people)
         {
@@ -586,12 +637,13 @@ getVigBuilder <- function(desc)
 
 getVigEngine <- function(vignetteFile){
     lines <- readLines(vignetteFile, n=100L, warn=FALSE)
-    idx <- grep(lines, pattern="VignetteEngine")
-    if (length(idx) != 0){
-        eng <- gsub("::.*", "", gsub(".*\\{|\\}.*", "", lines[idx]))
-        return(eng)
+    vigEngine <- grep(lines, pattern="VignetteEngine", value = TRUE)
+    vigEngine <- trimws(vigEngine)
+    if (length(vigEngine)) {
+        ve <- gsub("%\\s*\\\\VignetteEngine\\{(.*)\\}", "\\1", vigEngine)
+        head(strsplit(ve, "::", fixed = TRUE)[[1L]], 1L)
     } else {
-        return(NA)
+        NA
     }
 }
 
@@ -604,26 +656,26 @@ vigHelper <- function(vignetteFile, builder){
     }
 }
 
+.load_data <- function(dataname, package) {
+    env <- new.env(parent = emptyenv())
+    data(list = dataname, package = package, envir = env)
+    env[[dataname]]
+}
+
 getPkgType <- function(pkgdir)
 {
-    dcf <- read.dcf(file.path(pkgdir, "DESCRIPTION"))
-    if (!"biocViews" %in% colnames(dcf))
-    {
+    views <- .readBiocViews(pkgdir)
+    if (identical(length(views), 1L) && !nzchar(views))
         return(NA)
-    }
-    biocViews <- dcf[, "biocViews"]
-    views <- strsplit(gsub("\\s", "", biocViews), ",")[[1]]
-    biocViewsVocab <- NULL ## to keep R CMD check happy
-    data("biocViewsVocab", package="biocViews", envir=environment())
+
+    biocViewsVocab <- .load_data("biocViewsVocab", "biocViews")
+
     if (any(!views %in% nodes(biocViewsVocab)))
         return(NA)
-    parents <- c()
-    for (view in views)
-    {
-        parents <- c(parents, getParent(view, biocViewsVocab))
-    }
+
+    parents <- vapply(views, getParent, character(1L), biocViewsVocab)
     u <- unique(parents)
-    if (length(u)==1) return(u) else return(NA)
+    if (identical(length(u), 1L)) return(u) else return(NA)
 }
 
 getParent <- function(view, biocViewsVocab)
@@ -631,46 +683,14 @@ getParent <- function(view, biocViewsVocab)
     topLevel <- c("Software", "ExperimentData", "AnnotationData", "Workflow")
     if (view %in% topLevel)
         return(view)
+    parent <- ""
     for (level in topLevel) {
-        if (view %in% names(acc(biocViewsVocab, level)[[level]]))
-            return(level)
-    }
-}
-
-.checkEnv <- function(env, walker) {
-    ## look at all closures in 'env' using codetools-derived 'walker'
-    for (n in ls(env, all.names = TRUE)) {
-        v <- get(n, envir = env)
-        if (typeof(v) == "closure")
-            walkCode(body(v), walker)
-        else if (typeof(v) == "environment" && !walker$is_explored(v)) {
-            walker$mark_exploration(v)
-            .checkEnv(v, walker)
+        if (view %in% names(acc(biocViewsVocab, level)[[level]])) {
+            parent <- level
+            break
         }
     }
-    walker
-}
-
-.colonWalker <- function() {
-    ## record all pkg used as pkg::foo or pkg:::bar
-    PKGS <- character()
-    EXPLORED_ENVIRS <- character()
-    collector <- function(e, w)
-        PKGS <<- append(PKGS, as.character(e[[2]]))
-    mark_exploration <- function(env)
-        EXPLORED_ENVIRS <<- append(EXPLORED_ENVIRS, format(env))
-    list(handler=function(v, w) {
-        switch(v, "::"=collector, ":::"=collector, NULL)
-    }, call=function(e, w) {
-        for (ee in as.list(e)) if (!missing(ee)) walkCode(ee, w)
-    }, leaf = function(e, w) {
-        NULL
-    }, done = function() {
-        sort(unique(PKGS))
-    }, is_explored = function(env) {
-       format(env) %in% EXPLORED_ENVIRS
-    }, mark_exploration = mark_exploration
-    )
+    parent
 }
 
 getFunctionLengths <- function(df)
@@ -761,13 +781,11 @@ doesManPageHaveRunnableExample <- function(rd)
     length(parsed) && !inherits(parsed, "try-error")
 }
 
-selectSome <- function(obj, maxToShow = 5) {
-    stopifnot(is.character(obj))
-    if (!maxToShow %% 2)
-        stop("'maxToShow' should be an odd value")
-    edge <- (maxToShow - 1) / 2
-    if (length(obj) > maxToShow)
-        c(head(obj, edge), "...", tail(obj, edge))
-    else
-        obj
+.getYAMLfront <- function(lines) {
+    fm_idx <- grep("^---\\s*$", lines)
+    if (length(fm_idx) && !identical(length(fm_idx), 2L))
+        stop("More than 2 YAML front matter delimiters, i.e., '---' found")
+    if (length(fm_idx))
+        lines <- lines[seq(min(fm_idx), max(fm_idx))]
+    lines
 }
